@@ -1,9 +1,9 @@
-use inquire::{required, Text};
+use inquire::{length, required, ui::RenderConfig, validator::InquireLength, Text};
 
 fn main() {
     let answer = Text::new("What's your name?")
         .with_suggester(&suggester)
-        .with_validators(&[required!()])
+        .with_validators(&[required!(), length!(10)])
         .prompt()
         .unwrap();
 
@@ -12,11 +12,13 @@ fn main() {
     let _input = Text {
         message: "How are you feeling?",
         default: None,
+        placeholder: Some("Good"),
         help_message: None,
         formatter: Text::DEFAULT_FORMATTER,
         validators: Vec::new(),
         page_size: Text::DEFAULT_PAGE_SIZE,
         suggester: None,
+        render_config: RenderConfig::default_static_ref(),
     }
     .prompt()
     .unwrap();
