@@ -8,13 +8,55 @@ use crate::{
     date_utils::{get_current_date, get_month},
     error::{InquireError, InquireResult},
     formatter::{self, DateFormatter},
-    key::{Key, KeyModifiers},
-    renderer::Renderer,
-    terminal::Terminal,
+    ui::{
+        crossterm::CrosstermTerminal, date::DateSelectBackend, Backend, Key, KeyModifiers,
+        RenderConfig, Terminal,
+    },
     validator::DateValidator,
 };
 
-/// Interactive date picker to select simple dates (no time or timezones included).
+/// Prompt that allows user to select a date (time not supported) from an interactive calendar. Available via the `date` feature.
+///
+/// By default, the initial selected date is the current date. The user can navigate through the calendar by pressing the keyboard arrows. If the user also presses the control key along with the arrows, the user will be able to "fast-forward" to previous or next months or years.
+///
+/// More specifically:
+/// - Left arrow moves to the day previous to the one selected, and to the month previous to the one selected when pressed with `ctrl`.
+/// - Analogously, right arrow does the same, but moving to the next day or month.
+/// - Up arrow moves to the day above to the one selected, basically a week before the selected date. When pressed with `ctrl`, it moves to the previous year.
+/// - Analogously, the down arrow moves to a week later or a year later.
+///
+/// Finally, the user selects a date by pressing the space or enter keys.
+///
+/// `DateSelect` prompts provide several options of configuration:
+///
+/// - **Prompt message**: Required when creating the prompt.
+/// - **Default value**: Default value selected when the calendar is displayed and the one select if the user submits without any previous actions. Current date by default.
+/// - **Help message**: Message displayed at the line below the prompt.
+/// - **Formatter**: Custom formatter in case you need to pre-process the user input before showing it as the final answer.
+///   - Formats to "Month Day, Year" by default.
+/// - **Validators**: Custom validators to the user's selected date, displaying an error message if the date does not pass the requirements.
+/// - **Week start**: Which day of the week should be displayed in the first column of the calendar, Sunday by default.
+/// - **Min and max date**: Inclusive boundaries of allowed dates in the interactive calendar. If any boundary is set, the user will not be able to move past them, consequently not being able to select any dates out of the allowed range.
+///
+/// # Example
+///
+/// ```no_run
+/// use chrono::{NaiveDate, Weekday};
+/// use inquire::DateSelect;
+///
+/// let date = DateSelect::new("When do you want to travel?")
+///     .with_default(NaiveDate::from_ymd(2021, 8, 1))
+///     .with_min_date(NaiveDate::from_ymd(2021, 8, 1))
+///     .with_max_date(NaiveDate::from_ymd(2021, 12, 31))
+///     .with_week_start(Weekday::Mon)
+///     .with_help_message("Possible flights will be displayed according to the selected date")
+///     .prompt();
+///
+/// match date {
+///     Ok(_) => println!("No flights available for this date."),
+///     Err(_) => println!("There was an error in the system."),
+/// }
+/// ```
 #[derive(Clone)]
 pub struct DateSelect<'a> {
     /// Message to be presented to the user.
@@ -49,6 +91,9 @@ pub struct DateSelect<'a> {
     ///
     /// The possible error is displayed to the user one line above the prompt.
     pub validators: Vec<DateValidator<'a>>,
+
+    /// RenderConfig to apply to the rendered interface.
+    pub render_config: &'a RenderConfig,
 }
 
 impl<'a> DateSelect<'a> {
@@ -86,6 +131,7 @@ impl<'a> DateSelect<'a> {
             formatter: Self::DEFAULT_FORMATTER,
             validators: Self::DEFAULT_VALIDATORS,
             week_start: Self::DEFAULT_WEEK_START,
+            render_config: RenderConfig::default_static_ref(),
         }
     }
 
@@ -164,19 +210,25 @@ impl<'a> DateSelect<'a> {
         self
     }
 
+    /// Sets the provided color theme to this prompt.
+    pub fn with_render_config(mut self, render_config: &'a RenderConfig) -> Self {
+        self.render_config = render_config;
+        self
+    }
+
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to the defined rules.
     pub fn prompt(self) -> InquireResult<NaiveDate> {
-        let terminal = Terminal::new()?;
-        let mut renderer = Renderer::new(terminal)?;
-        self.prompt_with_renderer(&mut renderer)
+        let terminal = CrosstermTerminal::new()?;
+        let mut backend = Backend::new(terminal, self.render_config)?;
+        self.prompt_with_backend(&mut backend)
     }
 
-    pub(in crate) fn prompt_with_renderer(
+    pub(in crate) fn prompt_with_backend<T: Terminal>(
         self,
-        renderer: &mut Renderer,
+        backend: &mut Backend<T>,
     ) -> InquireResult<NaiveDate> {
-        DateSelectPrompt::new(self)?.prompt(renderer)
+        DateSelectPrompt::new(self)?.prompt(backend)
     }
 }
 
@@ -301,18 +353,18 @@ impl<'a> DateSelectPrompt<'a> {
         Ok(self.current_date)
     }
 
-    fn render(&mut self, renderer: &mut Renderer) -> InquireResult<()> {
+    fn render<B: DateSelectBackend>(&mut self, backend: &mut B) -> InquireResult<()> {
         let prompt = &self.message;
 
-        renderer.reset_prompt()?;
+        backend.frame_setup()?;
 
         if let Some(err) = &self.error {
-            renderer.print_error_message(err)?;
+            backend.render_error_message(err)?;
         }
 
-        renderer.print_prompt(&prompt, None, None)?;
+        backend.render_calendar_prompt(&prompt)?;
 
-        renderer.print_calendar_month(
+        backend.render_calendar(
             get_month(self.current_date.month()),
             self.current_date.year(),
             self.week_start,
@@ -323,21 +375,21 @@ impl<'a> DateSelectPrompt<'a> {
         )?;
 
         if let Some(help_message) = self.help_message {
-            renderer.print_help(help_message)?;
+            backend.render_help_message(help_message)?;
         }
 
-        renderer.flush()?;
+        backend.frame_finish()?;
 
         Ok(())
     }
 
-    fn prompt(mut self, renderer: &mut Renderer) -> InquireResult<NaiveDate> {
+    fn prompt<B: DateSelectBackend>(mut self, backend: &mut B) -> InquireResult<NaiveDate> {
         let final_answer: NaiveDate;
 
         loop {
-            self.render(renderer)?;
+            self.render(backend)?;
 
-            let key = renderer.read_key()?;
+            let key = backend.read_key()?;
 
             match key {
                 Key::Cancel => return Err(InquireError::OperationCanceled),
@@ -354,7 +406,7 @@ impl<'a> DateSelectPrompt<'a> {
 
         let formatted = (self.formatter)(final_answer);
 
-        renderer.cleanup(&self.message, &formatted)?;
+        backend.finish_prompt(&self.message, &formatted)?;
 
         Ok(final_answer)
     }
@@ -362,7 +414,11 @@ impl<'a> DateSelectPrompt<'a> {
 
 #[cfg(test)]
 mod test {
-    use crate::{date_utils::get_current_date, renderer::Renderer, terminal::Terminal, DateSelect};
+    use crate::{
+        date_utils::get_current_date,
+        ui::{crossterm::CrosstermTerminal, Backend, RenderConfig},
+        DateSelect,
+    };
     use chrono::NaiveDate;
     use crossterm::event::{KeyCode, KeyEvent};
     use ntest::timeout;
@@ -384,10 +440,11 @@ mod test {
                 let mut read = read.iter();
 
                 let mut write: Vec<u8> = Vec::new();
-                let terminal = Terminal::new_with_io(&mut write, &mut read);
-                let mut renderer = Renderer::new(terminal).unwrap();
+                let terminal = CrosstermTerminal::new_with_io(&mut write, &mut read);
+                let mut backend =
+                    Backend::new(terminal, RenderConfig::default_static_ref()).unwrap();
 
-                let ans = $prompt.prompt_with_renderer(&mut renderer).unwrap();
+                let ans = $prompt.prompt_with_backend(&mut backend).unwrap();
 
                 assert_eq!($output, ans);
             }
@@ -425,12 +482,12 @@ mod test {
         };
 
         let mut write: Vec<u8> = Vec::new();
-        let terminal = Terminal::new_with_io(&mut write, &mut read);
-        let mut renderer = Renderer::new(terminal).unwrap();
+        let terminal = CrosstermTerminal::new_with_io(&mut write, &mut read);
+        let mut backend = Backend::new(terminal, RenderConfig::default_static_ref()).unwrap();
 
         let ans = DateSelect::new("Question")
             .with_validator(&validator)
-            .prompt_with_renderer(&mut renderer)
+            .prompt_with_backend(&mut backend)
             .unwrap();
 
         assert_eq!(today_date.pred(), ans);
