@@ -14,6 +14,6 @@ pub use dateselect::DateSelect;
 
 pub use custom_type::CustomType;
 pub use multiselect::MultiSelect;
-pub use password::Password;
+pub use password::{Password, PasswordDisplayMode};
 pub use select::Select;
 pub use text::Text;
