@@ -1,11 +1,11 @@
-use std::{collections::HashSet, iter::FromIterator};
+use std::{collections::HashSet, fmt::Display};
 
 use crate::{
     config::{self, Filter},
     error::{InquireError, InquireResult},
-    formatter::{self, MultiOptionFormatter},
+    formatter::MultiOptionFormatter,
     input::Input,
-    option_answer::OptionAnswer,
+    list_option::ListOption,
     ui::{
         crossterm::CrosstermTerminal, Backend, Key, KeyModifiers, MultiSelectBackend, RenderConfig,
     },
@@ -17,7 +17,8 @@ use crate::{
 ///
 /// The user can select (or deselect) the current highlighted option by pressing space, clean all selections by pressing the left arrow and select all options by pressing the right arrow.
 ///
-/// This prompt requires a prompt message and a **non-empty** list of options to be displayed to the user. If the list is empty, the prompt operation will fail with an [`InquireError::InvalidConfiguration`] error.
+/// This prompt requires a prompt message and a **non-empty** `Vec` of options to be displayed to the user. The options can be of any type as long as they implement the `Display` trait. It is required that the `Vec` is moved to the prompt, as the prompt will return the ownership of the `Vec` after the user submits, with only the selected options inside it.
+/// - If the list is empty, the prompt operation will fail with an `InquireError::InvalidConfiguration` error.
 ///
 /// The options are paginated in order to provide a smooth experience to the user, with the default page size being 7. The user can move from the options and the pages will be updated accordingly, including moving from the last to the first options (or vice-versa).
 ///
@@ -41,13 +42,13 @@ use crate::{
 /// For a full-featured example, check the [GitHub repository](https://github.com/mikaelmello/inquire/blob/main/examples/multiselect.rs).
 ///
 /// [`InquireError::InvalidConfiguration`]: crate::error::InquireError::InvalidConfiguration
-#[derive(Copy, Clone)]
-pub struct MultiSelect<'a> {
+#[derive(Clone)]
+pub struct MultiSelect<'a, T> {
     /// Message to be presented to the user.
     pub message: &'a str,
 
     /// Options displayed to the user.
-    pub options: &'a [&'a str],
+    pub options: Vec<T>,
 
     /// Default indexes of options to be selected from the start.
     pub default: Option<&'a [usize]>,
@@ -67,30 +68,82 @@ pub struct MultiSelect<'a> {
 
     /// Function called with the current user input to filter the provided
     /// options.
-    pub filter: Filter<'a>,
+    pub filter: Filter<'a, T>,
 
     /// Whether the current filter typed by the user is kept or cleaned after a selection is made.
     pub keep_filter: bool,
 
     /// Function that formats the user input and presents it to the user as the final rendering of the prompt.
-    pub formatter: MultiOptionFormatter<'a>,
+    pub formatter: MultiOptionFormatter<'a, T>,
 
     /// Validator to apply to the user input.
     ///
     /// In case of error, the message is displayed one line above the prompt.
-    pub validator: Option<MultiOptionValidator<'a>>,
+    pub validator: Option<MultiOptionValidator<'a, T>>,
 
     /// RenderConfig to apply to the rendered interface.
     pub render_config: &'a RenderConfig,
 }
 
-impl<'a> MultiSelect<'a> {
-    /// Default formatter, set to [DEFAULT_MULTI_OPTION_FORMATTER](crate::formatter::DEFAULT_MULTI_OPTION_FORMATTER)
-    pub const DEFAULT_FORMATTER: MultiOptionFormatter<'a> =
-        formatter::DEFAULT_MULTI_OPTION_FORMATTER;
+impl<'a, T> MultiSelect<'a, T>
+where
+    T: Display,
+{
+    /// String formatter used by default in [MultiSelect](crate::MultiSelect) prompts.
+    /// Prints the string value of all selected options, separated by commas.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use inquire::list_option::ListOption;
+    /// use inquire::MultiSelect;
+    ///
+    /// let formatter = MultiSelect::<&str>::DEFAULT_FORMATTER;
+    ///
+    /// let mut ans = vec![ListOption::new(0, &"New York")];
+    /// assert_eq!(String::from("New York"), formatter(&ans));
+    ///
+    /// ans.push(ListOption::new(3, &"Seattle"));
+    /// assert_eq!(String::from("New York, Seattle"), formatter(&ans));
+    ///
+    /// ans.push(ListOption::new(7, &"Vancouver"));
+    /// assert_eq!(String::from("New York, Seattle, Vancouver"), formatter(&ans));
+    /// ```
+    pub const DEFAULT_FORMATTER: MultiOptionFormatter<'a, T> = &|ans| {
+        ans.iter()
+            .map(|opt| opt.to_string())
+            .collect::<Vec<String>>()
+            .join(", ")
+    };
 
-    /// Default filter, equal to the global default filter [config::DEFAULT_FILTER].
-    pub const DEFAULT_FILTER: Filter<'a> = config::DEFAULT_FILTER;
+    /// Default filter function, which checks if the current filter value is a substring of the option value.
+    /// If it is, the option is displayed.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use inquire::MultiSelect;
+    ///
+    /// let filter = MultiSelect::<&str>::DEFAULT_FILTER;
+    /// assert_eq!(false, filter("sa", &"New York",      "New York",      0));
+    /// assert_eq!(true,  filter("sa", &"Sacramento",    "Sacramento",    1));
+    /// assert_eq!(true,  filter("sa", &"Kansas",        "Kansas",        2));
+    /// assert_eq!(true,  filter("sa", &"Mesa",          "Mesa",          3));
+    /// assert_eq!(false, filter("sa", &"Phoenix",       "Phoenix",       4));
+    /// assert_eq!(false, filter("sa", &"Philadelphia",  "Philadelphia",  5));
+    /// assert_eq!(true,  filter("sa", &"San Antonio",   "San Antonio",   6));
+    /// assert_eq!(true,  filter("sa", &"San Diego",     "San Diego",     7));
+    /// assert_eq!(false, filter("sa", &"Dallas",        "Dallas",        8));
+    /// assert_eq!(true,  filter("sa", &"San Francisco", "San Francisco", 9));
+    /// assert_eq!(false, filter("sa", &"Austin",        "Austin",       10));
+    /// assert_eq!(false, filter("sa", &"Jacksonville",  "Jacksonville", 11));
+    /// assert_eq!(true,  filter("sa", &"San Jose",      "San Jose",     12));
+    /// ```
+    pub const DEFAULT_FILTER: Filter<'a, T> = &|filter, _, string_value, _| -> bool {
+        let filter = filter.to_lowercase();
+
+        string_value.to_lowercase().contains(&filter)
+    };
 
     /// Default page size, equal to the global default page size [config::DEFAULT_PAGE_SIZE]
     pub const DEFAULT_PAGE_SIZE: usize = config::DEFAULT_PAGE_SIZE;
@@ -108,11 +161,8 @@ impl<'a> MultiSelect<'a> {
     pub const DEFAULT_HELP_MESSAGE: Option<&'a str> =
         Some("↑↓ to move, space to select one, → to all, ← to none, type to filter");
 
-    /// Default validator set for the [MultiSelect] prompt, none.
-    pub const DEFAULT_VALIDATOR: Option<MultiOptionValidator<'a>> = None;
-
     /// Creates a [MultiSelect] with the provided message and options, along with default configuration values.
-    pub fn new(message: &'a str, options: &'a [&str]) -> Self {
+    pub fn new(message: &'a str, options: Vec<T>) -> Self {
         Self {
             message,
             options,
@@ -124,7 +174,7 @@ impl<'a> MultiSelect<'a> {
             keep_filter: Self::DEFAULT_KEEP_FILTER,
             filter: Self::DEFAULT_FILTER,
             formatter: Self::DEFAULT_FORMATTER,
-            validator: Self::DEFAULT_VALIDATOR,
+            validator: None,
             render_config: RenderConfig::default_static_ref(),
         }
     }
@@ -160,13 +210,13 @@ impl<'a> MultiSelect<'a> {
     }
 
     /// Sets the filter function.
-    pub fn with_filter(mut self, filter: Filter<'a>) -> Self {
+    pub fn with_filter(mut self, filter: Filter<'a, T>) -> Self {
         self.filter = filter;
         self
     }
 
     /// Sets the formatter.
-    pub fn with_formatter(mut self, formatter: MultiOptionFormatter<'a>) -> Self {
+    pub fn with_formatter(mut self, formatter: MultiOptionFormatter<'a, T>) -> Self {
         self.formatter = formatter;
         self
     }
@@ -176,7 +226,7 @@ impl<'a> MultiSelect<'a> {
     /// of selections.
     ///
     /// In case of error, the message is displayed one line above the prompt.
-    pub fn with_validator(mut self, validator: MultiOptionValidator<'a>) -> Self {
+    pub fn with_validator(mut self, validator: MultiOptionValidator<'a, T>) -> Self {
         self.validator = Some(validator);
         self
     }
@@ -201,7 +251,19 @@ impl<'a> MultiSelect<'a> {
 
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to the defined rules.
-    pub fn prompt(self) -> InquireResult<Vec<OptionAnswer>> {
+    ///
+    /// Returns the owned object selected by the user.
+    pub fn prompt(self) -> InquireResult<Vec<T>> {
+        self.raw_prompt()
+            .map(|op| op.into_iter().map(|o| o.value).collect())
+    }
+
+    /// Parses the provided behavioral and rendering options and prompts
+    /// the CLI user for input according to the defined rules.
+    ///
+    /// Returns a [`ListOption`](crate::list_option::ListOption) containing
+    /// the index of the selection and the owned object selected by the user.
+    pub fn raw_prompt(self) -> InquireResult<Vec<ListOption<T>>> {
         let terminal = CrosstermTerminal::new()?;
         let mut backend = Backend::new(terminal, self.render_config)?;
         self.prompt_with_backend(&mut backend)
@@ -210,14 +272,15 @@ impl<'a> MultiSelect<'a> {
     pub(in crate) fn prompt_with_backend<B: MultiSelectBackend>(
         self,
         backend: &mut B,
-    ) -> InquireResult<Vec<OptionAnswer>> {
+    ) -> InquireResult<Vec<ListOption<T>>> {
         MultiSelectPrompt::new(self)?.prompt(backend)
     }
 }
 
-struct MultiSelectPrompt<'a> {
+struct MultiSelectPrompt<'a, T> {
     message: &'a str,
-    options: &'a [&'a str],
+    options: Vec<T>,
+    string_options: Vec<String>,
     help_message: Option<&'a str>,
     vim_mode: bool,
     cursor_index: usize,
@@ -226,14 +289,17 @@ struct MultiSelectPrompt<'a> {
     keep_filter: bool,
     input: Input,
     filtered_options: Vec<usize>,
-    filter: Filter<'a>,
-    formatter: MultiOptionFormatter<'a>,
-    validator: Option<MultiOptionValidator<'a>>,
+    filter: Filter<'a, T>,
+    formatter: MultiOptionFormatter<'a, T>,
+    validator: Option<MultiOptionValidator<'a, T>>,
     error: Option<String>,
 }
 
-impl<'a> MultiSelectPrompt<'a> {
-    fn new(mso: MultiSelect<'a>) -> InquireResult<Self> {
+impl<'a, T> MultiSelectPrompt<'a, T>
+where
+    T: Display,
+{
+    fn new(mso: MultiSelect<'a, T>) -> InquireResult<Self> {
         if mso.options.is_empty() {
             return Err(InquireError::InvalidConfiguration(
                 "Available options can not be empty".into(),
@@ -251,23 +317,28 @@ impl<'a> MultiSelectPrompt<'a> {
             }
         }
 
+        let string_options = mso.options.iter().map(T::to_string).collect();
+        let filtered_options = (0..mso.options.len()).collect();
+        let checked_options = mso
+            .default
+            .map_or_else(|| HashSet::new(), |d| d.iter().cloned().collect());
+
         Ok(Self {
             message: mso.message,
             options: mso.options,
+            string_options,
+            filtered_options,
             help_message: mso.help_message,
             vim_mode: mso.vim_mode,
             cursor_index: mso.starting_cursor,
             page_size: mso.page_size,
             keep_filter: mso.keep_filter,
             input: Input::new(),
-            filtered_options: Vec::from_iter(0..mso.options.len()),
             filter: mso.filter,
             formatter: mso.formatter,
             validator: mso.validator,
             error: None,
-            checked: mso
-                .default
-                .map_or_else(|| HashSet::new(), |d| d.iter().cloned().collect()),
+            checked: checked_options,
         })
     }
 
@@ -277,24 +348,33 @@ impl<'a> MultiSelectPrompt<'a> {
             .enumerate()
             .filter_map(|(i, opt)| match self.input.content() {
                 val if val.is_empty() => Some(i),
-                val if (self.filter)(&val, opt, i) => Some(i),
+                val if (self.filter)(&val, opt, self.string_options.get(i).unwrap(), i) => Some(i),
                 _ => None,
             })
             .collect()
     }
 
-    fn move_cursor_up(&mut self) {
-        self.cursor_index = self
-            .cursor_index
-            .checked_sub(1)
-            .or(self.filtered_options.len().checked_sub(1))
-            .unwrap_or_else(|| 0);
+    fn move_cursor_up(&mut self, qty: usize, wrap: bool) {
+        if wrap {
+            let after_wrap = qty.saturating_sub(self.cursor_index);
+            self.cursor_index = self
+                .cursor_index
+                .checked_sub(qty)
+                .unwrap_or(self.filtered_options.len().saturating_sub(after_wrap))
+        } else {
+            self.cursor_index = self.cursor_index.saturating_sub(qty);
+        }
     }
 
-    fn move_cursor_down(&mut self) {
-        self.cursor_index = self.cursor_index.saturating_add(1);
+    fn move_cursor_down(&mut self, qty: usize, wrap: bool) {
+        self.cursor_index = self.cursor_index.saturating_add(qty);
+
         if self.cursor_index >= self.filtered_options.len() {
-            self.cursor_index = 0;
+            self.cursor_index = if wrap {
+                self.cursor_index % self.filtered_options.len()
+            } else {
+                self.filtered_options.len().saturating_sub(1)
+            }
         }
     }
 
@@ -317,10 +397,16 @@ impl<'a> MultiSelectPrompt<'a> {
 
     fn on_change(&mut self, key: Key) {
         match key {
-            Key::Up(KeyModifiers::NONE) => self.move_cursor_up(),
-            Key::Char('k', KeyModifiers::NONE) if self.vim_mode => self.move_cursor_up(),
-            Key::Down(KeyModifiers::NONE) => self.move_cursor_down(),
-            Key::Char('j', KeyModifiers::NONE) if self.vim_mode => self.move_cursor_down(),
+            Key::Up(KeyModifiers::NONE) => self.move_cursor_up(1, true),
+            Key::Char('k', KeyModifiers::NONE) if self.vim_mode => self.move_cursor_up(1, true),
+            Key::PageUp => self.move_cursor_up(self.page_size, false),
+            Key::Home => self.move_cursor_up(usize::MAX, false),
+
+            Key::Down(KeyModifiers::NONE) => self.move_cursor_down(1, true),
+            Key::Char('j', KeyModifiers::NONE) if self.vim_mode => self.move_cursor_down(1, true),
+            Key::PageDown => self.move_cursor_down(self.page_size, false),
+            Key::End => self.move_cursor_down(usize::MAX, false),
+
             Key::Char(' ', KeyModifiers::NONE) => self.toggle_cursor_selection(),
             Key::Right(KeyModifiers::NONE) => {
                 self.checked.clear();
@@ -353,25 +439,32 @@ impl<'a> MultiSelectPrompt<'a> {
         };
     }
 
-    fn get_final_answer(&self) -> Result<Vec<OptionAnswer>, String> {
-        let selected_options = self
-            .options
-            .iter()
-            .enumerate()
-            .filter_map(|(idx, opt)| match &self.checked.contains(&idx) {
-                true => Some(OptionAnswer::new(idx, opt)),
-                false => None,
-            })
-            .collect::<Vec<OptionAnswer>>();
-
+    fn validate_current_answer(&self) -> Result<(), String> {
         if let Some(validator) = self.validator {
-            return match validator(&selected_options) {
-                Ok(_) => Ok(selected_options),
-                Err(err) => Err(err),
-            };
-        }
+            let selected_options = self
+                .options
+                .iter()
+                .enumerate()
+                .filter_map(|(idx, opt)| match &self.checked.contains(&idx) {
+                    true => Some(ListOption::new(idx, opt)),
+                    false => None,
+                })
+                .collect::<Vec<ListOption<&T>>>();
 
-        return Ok(selected_options);
+            validator(&selected_options)
+        } else {
+            Ok(())
+        }
+    }
+
+    fn retain_final_answer(&mut self) {
+        let mut index = 0;
+        let checked = self.checked.clone();
+        self.options.retain(|_| {
+            let keep = checked.contains(&index);
+            index += 1;
+            keep
+        });
     }
 
     fn render<B: MultiSelectBackend>(&mut self, backend: &mut B) -> InquireResult<()> {
@@ -389,18 +482,12 @@ impl<'a> MultiSelectPrompt<'a> {
             .filtered_options
             .iter()
             .cloned()
-            .map(|i| OptionAnswer::new(i, self.options.get(i).unwrap()))
-            .collect::<Vec<OptionAnswer>>();
+            .map(|i| ListOption::new(i, self.options.get(i).unwrap()))
+            .collect::<Vec<ListOption<&T>>>();
 
         let page = paginate(self.page_size, &choices, self.cursor_index);
 
-        for (idx, opt) in page.content.iter().enumerate() {
-            backend.render_option(
-                &opt.value,
-                page.selection == idx,
-                self.checked.contains(&opt.index),
-            )?;
-        }
+        backend.render_options(page, &self.checked)?;
 
         if let Some(help_message) = self.help_message {
             backend.render_help_message(help_message)?;
@@ -414,9 +501,7 @@ impl<'a> MultiSelectPrompt<'a> {
     fn prompt<B: MultiSelectBackend>(
         mut self,
         backend: &mut B,
-    ) -> InquireResult<Vec<OptionAnswer>> {
-        let final_answer: Vec<OptionAnswer>;
-
+    ) -> InquireResult<Vec<ListOption<T>>> {
         loop {
             self.render(backend)?;
 
@@ -424,18 +509,25 @@ impl<'a> MultiSelectPrompt<'a> {
 
             match key {
                 Key::Cancel => return Err(InquireError::OperationCanceled),
-                Key::Submit => match self.get_final_answer() {
-                    Ok(answer) => {
-                        final_answer = answer;
-                        break;
-                    }
+                Key::Submit => match self.validate_current_answer() {
+                    Ok(()) => break,
                     Err(err) => self.error = Some(err),
                 },
                 key => self.on_change(key),
             }
         }
 
-        let formatted = (self.formatter)(&final_answer);
+        self.retain_final_answer();
+
+        let final_answer: Vec<ListOption<T>> = self
+            .options
+            .into_iter()
+            .enumerate()
+            .map(|(idx, opt)| ListOption::new(idx, opt))
+            .collect();
+
+        let refs: Vec<ListOption<&T>> = final_answer.iter().map(ListOption::as_ref).collect();
+        let formatted = (self.formatter)(&refs);
 
         backend.finish_prompt(&self.message, &formatted)?;
 
