@@ -5,7 +5,7 @@ use crate::{
     error::{InquireError, InquireResult},
     formatter::{StringFormatter, DEFAULT_STRING_FORMATTER},
     input::Input,
-    option_answer::OptionAnswer,
+    list_option::ListOption,
     ui::{crossterm::CrosstermTerminal, Backend, Key, KeyModifiers, RenderConfig, TextBackend},
     utils::paginate,
     validator::StringValidator,
@@ -264,21 +264,25 @@ impl<'a> TextPrompt<'a> {
         }
     }
 
-    fn move_cursor_up(&mut self) {
-        self.cursor_index = self.cursor_index.saturating_sub(1);
+    fn move_cursor_up(&mut self, qty: usize) {
+        self.cursor_index = self.cursor_index.saturating_sub(qty);
     }
 
-    fn move_cursor_down(&mut self) {
+    fn move_cursor_down(&mut self, qty: usize) {
         self.cursor_index = min(
-            self.cursor_index.saturating_add(1),
+            self.cursor_index.saturating_add(qty),
             self.suggested_options.len(),
         );
     }
 
     fn on_change(&mut self, key: Key) {
         match key {
-            Key::Up(KeyModifiers::NONE) => self.move_cursor_up(),
-            Key::Down(KeyModifiers::NONE) => self.move_cursor_down(),
+            Key::Up(KeyModifiers::NONE) => self.move_cursor_up(1),
+            Key::PageUp => self.move_cursor_up(self.page_size),
+
+            Key::Down(KeyModifiers::NONE) => self.move_cursor_down(1),
+            Key::PageDown => self.move_cursor_down(self.page_size),
+
             key => {
                 let dirty = self.input.handle_key(key);
 
@@ -345,17 +349,19 @@ impl<'a> TextPrompt<'a> {
             .suggested_options
             .iter()
             .enumerate()
-            .map(|(i, val)| OptionAnswer::new(i, val))
-            .collect::<Vec<OptionAnswer>>();
+            .map(|(i, val)| ListOption::new(i, val.as_ref()))
+            .collect::<Vec<ListOption<&str>>>();
 
-        let display_cursor = self.cursor_index > 0;
         let list_index = self.cursor_index.saturating_sub(1);
-        let page = paginate(self.page_size, &choices, list_index);
+        let mut page = paginate(self.page_size, &choices, list_index);
 
-        for (idx, opt) in page.content.iter().enumerate() {
-            backend.render_suggestion(&opt.value, display_cursor && page.selection == idx)?;
+        let cursor_on_input = self.cursor_index == 0;
+        if cursor_on_input {
+            page.selection = usize::MAX;
         }
 
+        backend.render_suggestions(page)?;
+
         if let Some(message) = self.help_message {
             backend.render_help_message(message)?;
         } else if !choices.is_empty() {
